package com.kilo.ecs;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;

import android.os.SystemClock;

import com.kilo.bean.LogBean;
import com.kilo.util.LoadConfig;

/**
 * @author chifengliang
 *
 */
public abstract class BasePrint extends WriteFile
{
    private static final String TAG = "BasePrint";
    private static final byte[] lock = new byte[0];
    public BasePrint(String filePath, int max_size)
    {
        super(filePath, max_size);
        Executors.newCachedThreadPool().execute(new WriteThread());
        LoadConfig.getInstance().loadConfigFile(filePath);
    }
    private BlockingQueue<LogBean> printQueue = new LinkedBlockingQueue<LogBean>();
    private SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
    
    public synchronized void addTask(LogLevel level, String tag, String msg)
    {
        long time = (new Date()).getTime();
        String timeStr = sdf.format(new Date());
        LogBean bean = new LogBean();
        LogBean.BEANCOUNT++;
        bean.seq = LogBean.BEANCOUNT;
        bean.timeStr = timeStr;
        bean.time = time / 1000;
        bean.tag = tag;
        bean.className = getFunName().split("[.]")[0];
        bean.level = level;
        bean.msg = msg;
        bean.func = getFunName().split("[.]")[1];
        bean.line = getLine();
        printQueue.add(bean);
        unlock();
    }
    
    private String getFunName()
    {
        StackTraceElement t = new Throwable().getStackTrace()[5];
        String cls = t.getClassName();
        int clsInd = cls.lastIndexOf(".");
        cls = cls.substring(clsInd == -1 ? 0 : clsInd + 1);
        return cls + '.' + t.getMethodName();
    }
    
    private int getLine()
    {
        StackTraceElement t = new Throwable().getStackTrace()[5];
        int line = t.getLineNumber();
        return line;
    }
    
    public abstract void doTask(LogBean bean);
    
    private void lock()
    {
        synchronized (lock)
        {
            try
            {
                lock.wait();
            }
            catch (InterruptedException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
    }
    private void unlock()
    {
        synchronized (lock)
        {
            lock.notifyAll();
        }
    }
    /**
     * @author kilo
     *
     */
    private class WriteThread implements Runnable
    {
        @Override
        public void run()
        {
            // TODO Auto-generated method stub
            while (true)
            {
                if (!printQueue.isEmpty())
                {
                    try
                    {
                        LogBean logBean = printQueue.take();
                        doTask(logBean);
                        write(logBean.toString());
                        SystemClock.sleep(1);
                    }
                    catch (InterruptedException e)
                    {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                }
                else
                {
                    lock();
                }
            }
        }
    }
}
