package com.kilo.ecs;

import android.content.Context;
import android.util.Log;

import com.kilo.util.LoadConfig;

/**
 *  日志输出类
 *  可将所有的日志输出到指定的目录中.
 *  如果有可用服务器，并上传 (调用initServerInfo()设置服务)
 * @author kilo
 *
 */
public final class ECS
{
    public static String LOG_FILE_NAME = "ECS.txt";
    private PrintTask printTask = null;
    /**
     * @param filePath
     * @param max_size
     */
    public ECS(String filePath, int max_size)
    {
        if (null != printTask)
        {
            return;
        }
        printTask = new PrintTask(filePath, max_size);
        
        StringBuffer sb = new StringBuffer("\n");
        sb.append("**********************************************************************\n")
          .append("*********************  ECS version 2.0.1  ****************************\n")
          .append("**********************************************************************\n");
        print(LogLevel.INFO,"",sb.toString());
    }
    public ECS(String filePath)
    {
        this(filePath, 0xA00000);
    }
    
    public void d(String tag, String msg)
    {
        if (LoadConfig.getInstance().getConfig().debug)
        {
            Log.d(tag, msg);
        }
        print(LogLevel.DEBUG, tag, msg);
    }
    
    public void e(String tag, String msg)
    {
        if (LoadConfig.getInstance().getConfig().debug)
        {
            Log.e(tag, msg);
        }
        print(LogLevel.ERROR, tag, msg);
    }
    
    public void i(String tag, String msg)
    {
        if (LoadConfig.getInstance().getConfig().debug)
        {
            Log.i(tag, msg);
        }
        print(LogLevel.INFO, tag, msg);
    }
    
    public void w(String tag, String msg)
    {
        if (LoadConfig.getInstance().getConfig().debug)
        {
            Log.w(tag, msg);
        }
        print(LogLevel.WORNING, tag, msg);
    }
    
    private void print(LogLevel level, String tag, String msg)
    {
        printTask.addTask(level, tag, msg);
    }
    
    /**
     * 如果注册和上传服务为同一个请写一个即可
     * @param context
     */
    public void initServerInfo(final Context context)
    {
        printTask.initServerInfo(context);
    }
}
