package com.kilo.ecs;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.RandomAccessFile;

/**
 * @author chifengliang
 *
 */
public class WriteFile
{
    private String filePath = "";
    private long max_size = 0xA00000;
    private boolean createFile = true;
    public WriteFile(String filePath, int max_size)
    {
        this.filePath = filePath;
        this.max_size = max_size;
    }
    
    /**
     * @param sb
     */
    protected void write(String sb)
    {
        File file = checkFile(filePath);
        if (null == file)
        {
            return;
        }
        try
        {
            RandomAccessFile raf = new RandomAccessFile(file, "rw");
            raf.seek(file.length());
            raf.write(sb.getBytes());
            raf.close();
        }
        catch (FileNotFoundException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
    
    /**
     * @param filePath
     * @return
     */
    private File checkFile(String filePath)
    {
        if (null == filePath || "".equals(filePath) || !createFile)
        {
            return null;
        }
        String fileName = filePath.endsWith("/") ? filePath + ECS.LOG_FILE_NAME : filePath + "/" + ECS.LOG_FILE_NAME;
        File file = new File(fileName);
        if (!file.exists())
        {
            try
            {
                if (!file.getParentFile().exists())
                {
                    file.getParentFile().mkdirs();
                }
                boolean result = file.createNewFile();
                if (!result)
                {
                    return null;
                }
            }
            catch (IOException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
                createFile = false;
            }
        }
        long size = file.length();
        if (size > max_size)
        {
            file = createCpyFile(file, true);
        }
        if (file.exists())
        {
            return file;
        }
        return null;
    }
    
    /**
     * @param oldFile
     * @return
     */
    public static File createCpyFile(File oldFile, boolean deleteOld)
    {
        if (null == oldFile)
        {
            return null;
        }
        File newFile = null;
        try
        {
//            newFile = File.createTempFile("", ".bak", oldFile);
            newFile = new File(oldFile.getCanonicalPath() + ".bak");
            if (newFile.exists())
            {
                newFile.delete();
            }
            copyFile(oldFile, newFile);
            if (deleteOld)
            {
                oldFile.delete();
            }
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return oldFile; 
    }
    
    public static void inputStreamToFile(InputStream is, File file)
    {
        if (null == is || null == file)
        {
            return;
        }
        BufferedReader br = null;
        BufferedWriter bw = null;
        String line = "";
        try
        {
            br = new BufferedReader(new InputStreamReader(is));
            bw = new BufferedWriter(new FileWriter(file));
            while ((line = br.readLine()) != null)
            {
                bw.write(line);
            }
            bw.flush();
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        finally
        {
            closeBufferedFileReader(br);
            closeBufferedFileWriter(bw);
        }
    }
    
    /**
     * @param oldFile
     * @param newFile
     */
    public static void copyFile(File oldFile, File newFile)
    {
        if (null == oldFile || null == newFile)
        {
            return;
        }
        try
        {
            inputStreamToFile(new FileInputStream(oldFile), newFile);
        }
        catch (FileNotFoundException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
//        BufferedWriter fw = null;
//        BufferedReader fr = null;
//        String line = "";
//        try
//        {
//            fw = new BufferedWriter(new FileWriter(newFile));
//            fr = new BufferedReader(new FileReader(oldFile));
//            while ((line = fr.readLine()) != null)
//            {
//                fw.write(line);
//            }
//            fw.flush();
//        }
//        catch (IOException e)
//        {
//            // TODO Auto-generated catch block
//            e.printStackTrace();
//        }
//        finally
//        {
//            closeBufferedFileReader(fr);
//            closeBufferedFileWriter(fw);
//        }
        
    }
    
    public static File createFile(String file)
    {
        if (null == file || "".equals(file))
        {
            return null;
        }
        File createFile = new File(file);
        if (createFile.exists())
        {
            return createFile;
        }
        File parent = createFile.getParentFile();
        if (!parent.exists())
        {
            parent.mkdir();
        }
        try
        {
            createFile.createNewFile();
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return createFile;
    }
    
    /**
     * @param bw
     */
    private static void closeBufferedFileWriter(BufferedWriter bw)
    {
        if (null != bw)
        {
            try
            {
                bw.close();
            }
            catch (IOException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
    }
    
    private static void closeBufferedFileReader(BufferedReader br)
    {
        if (null != br)
        {
            try
            {
                br.close();
            }
            catch (IOException e)
            {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
        }
    }
}
