package com.kilo.upload;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.Executors;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.DefaultHttpClient;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.provider.Settings.Secure;
import android.telephony.TelephonyManager;

import com.kilo.bean.AppBean;
import com.kilo.bean.LogBean;
import com.kilo.ecs.LogLevel;
import com.kilo.ecs.PrintTask;
import com.kilo.listener.ConfigListener;
import com.kilo.util.Config;
import com.kilo.util.LoadConfig;

/**
 * @author chifengliang
 * http post方式
 */
public class HttpUpload extends UpLoad implements ConfigListener
{
    private static final String TAG = "HttpUpload";
//    private Context mContext;
//    private static final String TAG = "HttpUpload";
    private static final int waitTime = 1000;
    private static final int MAX_REGIST_COUNT = 20;
//    private HttpClient client = new DefaultHttpClient();
    private Context mContext;
    public HttpUpload()
    {
    }
    
    /**
     * 发起注册
     * 最大注册次数<code>${MAX_REGIST_COUNT}</code>
     * @param url
     * @param context
     */
    @Override
    public void initServerInfo(final Context context)
    {
        mContext = context;
        if (null == context || null == LoadConfig.getInstance().getConfig().registServer
            || "".equals(LoadConfig.getInstance().getConfig().registServer)
            || null == LoadConfig.getInstance().getConfig().upLoadlogServer
            || "".equals(LoadConfig.getInstance().getConfig().upLoadlogServer)
            || !LoadConfig.getInstance().getConfig().debug)
        {
//            log.addTask(LogLevel.ERROR, TAG, "config fail not regist server.");
            return;
        }
        registURL = LoadConfig.getInstance().getConfig().registServer;
        upLoadURL = LoadConfig.getInstance().getConfig().upLoadlogServer;
        if (isRegist)
        {
            unLock();
            return;
        }
        Executors.newSingleThreadExecutor().execute(new Runnable()
        {
            @Override
            public void run()
            {
                // TODO Auto-generated method stub
                int count = 1;
                do
                {
//                    log.addTask(LogLevel.ERROR, TAG, "test regist " + count);
                    isRegist = registInPool(registURL, context);
                    if (!isRegist)
                    {
                        // 如果一次没有注册成功等待一段时间再注册
                        lock(count * waitTime);
                    }
                    else
                    {
                        unLock();
                        break;
                    }
                    if (count++ > MAX_REGIST_COUNT)
                    {
                        break;
                    }
                }while (!isRegist);
//                log.addTask(LogLevel.ERROR, TAG, "regist ret=" + isRegist);
                if (isRegist)
                {
                    afterRegistSuccess();
                }
            }
        });
    }
    
    private boolean registInPool(String url, Context context)
    {
        this.registURL = url;
        AppBean app = getAppInfo(context);
        HttpClient client = new DefaultHttpClient();
//        HttpUriRequest request = new HttpPost(url);
        HttpPost request = new HttpPost(url);
        InputStream in = null;
        BufferedOutputStream out = null;
        ByteArrayOutputStream dataStream = null;
        HttpResponse response = null;
        HttpEntity upEntity = null;
        try
        {
            upEntity = new StringEntity(getRegistXML(app));
            request.setEntity(upEntity);
            response = client.execute(request);
            HttpEntity entity = response.getEntity();
            if (entity == null)
            {
                return false;
            }
            in = entity.getContent();
            dataStream = new ByteArrayOutputStream();
            out = new BufferedOutputStream(dataStream, BUFFER_SIZE);
            copy(in, out);
            out.flush();
            String session = dataStream.toString().split(" ")[0];
            LogBean.SESSION = session;
            return true;
        }
        catch (ClientProtocolException e)
        {
//            log.addTask(LogLevel.ERROR, TAG, e.toString());
        }
        catch (IOException e)
        {
//            log.addTask(LogLevel.ERROR, TAG, e.toString());
        }
        finally
        {
            closeStream(dataStream);
            closeStream(out);
            closeStream(in);
        }
        return false;
    }
    
    /**
     * 组装注册的xml
     * @param app
     * @return
     */
    private String getRegistXML(AppBean app)
    {
        StringBuffer sb = new StringBuffer(XML_HEAD).append("\n");
        sb.append("<body>").append("\n")
          .append("<App>").append("\n")
          .append("<build>").append(encodeBase64(app.appBuild)).append("</build>").append("\n")
          .append("<id>").append(encodeBase64(app.appId)).append("</id>").append("\n")
          .append("<displayName>").append(encodeBase64(app.appDisplayName)).append("</displayName>").append("\n")
          .append("<version>").append(encodeBase64(app.appVersion)).append("</version>").append("\n")
          .append("</App>").append("\n")
          .append("<Device>").append("\n")
          .append("<id>").append(encodeBase64(app.deviceId)).append("</id>").append("\n")
          .append("<model>").append(encodeBase64(app.deviceModel)).append("</model>").append("\n")
          .append("<name>").append(encodeBase64(app.deviceName)).append("</name>").append("\n")
          .append("<os_version>").append(encodeBase64(app.os_version)).append("</os_version>").append("\n")
          .append("</Device>").append("\n")
          .append("</body>").append("\n");
        return sb.toString();
    }
    
    /**
     * 组装上传log数据的xml形式
     * @param log
     * @return
     */
    private String getUpLoadLogXML(List<LogBean> logs)
    {
        if (null == logs || logs.isEmpty())
        {
            return "";
        }
        StringBuffer sb = new StringBuffer(XML_HEAD).append("\n");
        sb.append("<body>").append("\n")
          .append("<session>").append(LogBean.SESSION).append("</session>").append("\n")
          .append("<content>").append("\n");
        for (LogBean log : logs)
        {
            sb.append("<log>").append("\n")
            .append("<seq>").append(encodeBase64(log.seq + "")).append("</seq>").append("\n")
            .append("<time>").append(encodeBase64(log.time + "")).append("</time>").append("\n")
            .append("<text>").append(encodeBase64(log.msg)).append("</text>").append("\n")
            .append("<file>").append(encodeBase64(log.className)).append("</file>").append("\n")
            .append("<line>").append(encodeBase64(log.line + "")).append("</line>").append("\n")
            .append("<tag>").append(encodeBase64(log.tag)).append("</tag>").append("\n")
            .append("<func>").append(encodeBase64(log.func)).append("</func>").append("\n")
            .append("</log>").append("\n");
        }
        sb.append("</content>").append("\n")
          .append("</body>").append("\n");
        return sb.toString();
    }
    
    /**
     * @param context
     * @return
     */
    private AppBean getAppInfo(Context context)
    {
//        mContext = context;
        PackageManager pm = context.getPackageManager();
        AppBean app = new AppBean();
        try
        {
            app.deviceModel = android.os.Build.MODEL;
            app.deviceId = android.os.Build.ID;
            app.os_version = android.os.Build.VERSION.SDK;
            app.deviceName = android.os.Build.DEVICE;
            
            // 0表示获取版本信息
            PackageInfo packInfo = pm.getPackageInfo(context.getPackageName(), 0);
            String version = packInfo.versionName;
            app.appVersion = version;
            app.appDisplayName = packInfo.applicationInfo.loadLabel(pm).toString();
            app.appBuild = app.appVersion;
            app.appId = context.getPackageName();
            WifiManager wifiMgr = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
            WifiInfo info = (null == wifiMgr ? null : wifiMgr.getConnectionInfo());
            String macAddress = null;
            if (null != info) {
                macAddress = info.getMacAddress();
            }

            if (macAddress == null || "".equals(macAddress)) {
                macAddress = getUUID(context);
            }
//            app.deviceId = macAddress;
        }
        catch (NameNotFoundException e)
        {
//            log.addTask(LogLevel.ERROR, TAG, e.toString());
        }
        return app;
    }
    
    /**
     * 
     * @param context
     * @return String
     * @since V1.0
     */
    public String getUUID(Context context) {
        String uuid = "";
        synchronized (HttpUpload.class) {
            final String androidId = Secure.getString(context.getContentResolver(), Secure.ANDROID_ID);
            try {
                if (!"9774d56d682e549c".equals(androidId)) {
                    uuid = UUID.nameUUIDFromBytes(androidId.getBytes("utf8")).toString();
                } else {
                    final String deviceId = ((TelephonyManager) context.getSystemService(Context.TELEPHONY_SERVICE))
                            .getDeviceId();
                    uuid = deviceId != null ? UUID.nameUUIDFromBytes(deviceId.getBytes("utf8")).toString() : UUID
                            .randomUUID().toString();
                }
            } catch (UnsupportedEncodingException e) {
                throw new RuntimeException(e);
            }
        }

        return uuid;
    }

    
    @Override
    int doUpLoadLog(List<LogBean> logs)
    {
        if ("".equals(upLoadURL) || null == logs || logs.isEmpty())
        {
            return 0;
        }
        // TODO Auto-generated method stub
        HttpClient client = new DefaultHttpClient();
        HttpPost request = new HttpPost(upLoadURL);
//        InputStream in = null;
//        BufferedOutputStream out = null;
//        ByteArrayOutputStream dataStream = null;
        HttpResponse response = null;
        HttpEntity upEntity = null;
        try
        {
            upEntity = new StringEntity(getUpLoadLogXML(logs));
            request.setEntity(upEntity);
            response = client.execute(request);
//            HttpEntity entity = response.getEntity();
//            if (entity == null)
//            {
//                return -1;
//            }
//            in = entity.getContent();
//            if (null == in)
//            {
//                return -1;
//            }
//            dataStream = new ByteArrayOutputStream();
//            out = new BufferedOutputStream(dataStream, BUFFER_SIZE);
//            copy(in, out);
//            out.flush();
            if (response.getStatusLine().getStatusCode() == 200)
            {
                return 0;
            }
            else
            {
                return -1;
            }
        }
        catch (ClientProtocolException e)
        {
//            log.addTask(LogLevel.ERROR, TAG, e.toString());
        }
        catch (IOException e)
        {
//            log.addTask(LogLevel.ERROR, TAG, e.toString());
        }
        finally
        {
//            upEntity = null;
//            closeOutputStream(dataStream);
//            closeOutputStream(out);
//            closeInputStream(in);
        }
        return -1;
    }

    @Override
    public void configChange(Config config)
    {
        // TODO Auto-generated method stub
        if (null != mContext)
        {
            isRegist = false;
            initServerInfo(mContext);
        }
    }
}
