package com.kilo.upload;

import java.io.Closeable;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;

import android.content.Context;

import com.kilo.bean.LogBean;
import com.kilo.ecs.LogLevel;
import com.kilo.ecs.PrintTask;

/**
 * @author chifengliang
 *
 */
public abstract class UpLoad
{
    protected static final String TAG = UpLoad.class.getName();
    /**1024*8*/
    public static final int BUFFER_SIZE = 0x002000;
    private static final String BASE64_ENC = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    protected static final byte[] LOCK = new byte[0];
    protected static final String XML_HEAD = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>";
    protected boolean isRegist = false;
    protected String registURL = "";
    protected String upLoadURL = "";
    private BlockingQueue<LogBean> logQueue = new LinkedBlockingQueue<LogBean>();
    private List<LogBean> sendLogList = new ArrayList<LogBean>();
    public UpLoad()
    {
    }
    
    public void send(LogBean logBean)
    {
        logQueue.add(logBean);
    }
    
    protected void afterRegistSuccess()
    {
        Executors.newCachedThreadPool().execute(new UpLoadLogThread());
    }
    
    public abstract void initServerInfo(Context context);
    
    /**
     * 上传数据
     * @param logs
     * return 最后一条的id
     */
    abstract int doUpLoadLog(List<LogBean> logs);
    
    /**
     * @param in src
     * @param out target
     * @return copy length
     * @throws IOException 
     */
    public long copy(final InputStream in, final OutputStream out) throws IOException
    {
        long length = 0;
        final byte[] b = new byte[BUFFER_SIZE];
        int read;
        while ((read = in.read(b)) != -1) {
            out.write(b, 0, read);
            length += read;
        }
        return length;
    }
    
    /**
     * @return true 连通
     */
    public boolean isRegist()
    {
        return isRegist;
    }
    
    protected void closeStream(Closeable stream)
    {
        if (null != stream)
        {
            try
            {
                stream.close();
            }
            catch (IOException e)
            {
            }
        }
    }
    
    protected static String encodeBase64(final String s) {
        if (null == s)
        {
            return "";
        }
        final StringBuilder out = new StringBuilder();
        try
        {
            for (int i = 0; i < (s.length() + 2) / 3; ++i) {
                final int l = Math.min(3, s.length() - i * 3);
                final String buf = s.substring(i * 3, i * 3 + l);
                out.append(BASE64_ENC.charAt(buf.charAt(0) >> 2));
                out.append(BASE64_ENC.charAt((buf.charAt(0) & 0x03) << 4
                        | (l == 1 ? 0 : (buf.charAt(1) & 0xf0) >> 4)));
                out.append(l > 1 ? BASE64_ENC.charAt((buf.charAt(1) & 0x0f) << 2
                        | (l == 2 ? 0 : (buf.charAt(2) & 0xc0) >> 6)) : '=');
                out.append(l > 2 ? BASE64_ENC.charAt(buf.charAt(2) & 0x3f) : '=');
            }
        }
        catch (Exception e){}
        return out.toString();
    }
    
    /**
     * @author kilo
     *
     */
    private class UpLoadLogThread implements Runnable
    {
        @Override
        public void run()
        {
            // TODO Auto-generated method stub
            while (true)
            {
                if (!isRegist)
                {
                    lock(0);
                }
                if (null == upLoadURL || "".equals(upLoadURL))
                {
                    lock(0);
                }
                int size = logQueue.size();
                size = size > 5 ? 5 : size;//一次读5行，如果有的话
                for (int i = 0; i < size; i++)
                { 
                    if (!logQueue.isEmpty())
                    {
                        try
                        {
                            sendLogList.add(logQueue.take());
                        }catch (InterruptedException e){}
                    }
                    else
                    {
                        break;
                    }
                }
                // 如果没有上传成功，重新上传
                while(doUpLoadLog(sendLogList) != 0){sleep(500);};
                if (sendLogList.size() > 0)
                {
                    LogBean.SET_SEQ_SP(sendLogList.get(sendLogList.size() - 1).seq);
                }
                sendLogList.clear();
                sleep(10);
            }
        }
    }
    
    /**
     * 锁time时间
     * @param time
     */
    protected void lock(long time)
    {
        if (0 >= time)
        {
            try
            {
                synchronized (LOCK)
                {
                    LOCK.wait();
                    return;
                }
            }
            catch (InterruptedException e)
            {
            }
        }
        try
        {
            synchronized (LOCK)
            {
                LOCK.wait(time);
            }
        }
        catch (InterruptedException e)
        {
        }
    }
    
    protected void unLock()
    {
        synchronized (LOCK)
        {
            LOCK.notifyAll();
        }
    }
    
    private void sleep(long time)
    {
        try
        {
            Thread.sleep(time);
        }
        catch (InterruptedException e)
        {
        }
    }
}
