package com.kilo.util;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.concurrent.Executors;

import android.util.Log;

import com.kilo.ecs.WriteFile;
import com.kilo.listener.ConfigListener;

public class LoadConfig
{
    private static final String TAG = "LoadConfig";
    private static final String FILE_PATH = "/assets/";
    private static final String FILE_NAME = "config.properties";
    private Properties properties;
    private Config config;
    private static LoadConfig instance;
    private String filePath = "";
    private List<ConfigListener> listeners = new ArrayList<ConfigListener>();
    private long lastModified;
    private Thread modifyListener;
    private LoadConfig()
    {
        config = new Config();
        properties = new Properties();
        loadConfigFile(FILE_PATH);
    }
    
    public static LoadConfig getInstance()
    {
        if (null == instance)
        {
            instance = new LoadConfig();
        }
        return instance;
    }
    
    /**
     * 加载文件
     * @param file
     * @return
     */
    public boolean loadConfigFile(String file)
    {
        boolean result = false;
        if (null == file || "".equals(file))
        {
            return false;
        }
        if (!file.endsWith("/"))
        {
            file += "/"; 
        }
        if (file.equals(FILE_PATH))
        {
            file += FILE_NAME;
        }
        else
        {
            file += "config/" + FILE_NAME;
        }
        filePath = file;
        Log.d(TAG, "file=" + file);
        InputStream is = null;
        try
        {
            is = LoadConfig.class.getResourceAsStream(file);
            if (null == is)
            {
                // 拷贝系统的文件
                is = LoadConfig.class.getResourceAsStream(FILE_PATH + FILE_NAME);
                final InputStream instream = is;
                final String fileStr = file;
                Executors.newCachedThreadPool().execute(new Runnable()
                {
                    @Override
                    public void run()
                    {
                        // TODO Auto-generated method stub
                        WriteFile.inputStreamToFile(instream, WriteFile.createFile(fileStr));
                    }
                });
                return false;
            }
            properties.load(is);
            String debugStr = properties.getProperty("debug", "false").trim().toLowerCase();
            if (debugStr.equals("true") || debugStr.equals("t"))
            {
                config.debug = true;
            }
//            config.registServer = properties.getProperty("regist_ip_addr", "http://10.17.36.119:5195/Background/register.php");
//            config.upLoadlogServer = properties.getProperty("send_log_ip", "http://10.17.36.119:5195/Background/reportLog.php");
            config.registServer = properties.getProperty("regist_ip_addr", "");
            config.registServer = properties.getProperty("send_log_ip", "");
            result = true;
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
            result = false;
        }
        
        if (!filePath.equals(FILE_PATH + FILE_NAME))
        {
            lastModified = (new File(filePath)).lastModified();
            if (null == modifyListener)
            {
                modifyListener = new Thread(new WatchFileModifyListener());
                modifyListener.start();
                Log.d(TAG, "file listener start");
            }
        }
        return result;
    }
    
    public Config getConfig()
    {
        return config;
    }
    
    /**
     * 注册监听事件
     * @param listener
     */
    public void regConfigChange(ConfigListener listener)
    {
        if (!listeners.contains(listener))
        {
            return;
        }
        listeners.add(listener);
    }
    
    /**
     * 解注册监听事件
     * @param listener
     */
    public void unRegConfigChange(ConfigListener listener)
    {
        if (listeners.contains(listener))
        {
            listeners.remove(listener);
        }
    }
    
    /**
     * 
     */
    private void notifyConfigChange()
    {
        if (listeners.isEmpty())
        {
            return;
        }
        for (ConfigListener listener : listeners)
        {
            listener.configChange(config);
        }
    }
    
    private void sleep(long time)
    {
        try
        {
            Thread.sleep(time);
        }
        catch (InterruptedException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
    
    /**
     * 监听文件修改
     * @author chifengliang
     *
     */
    protected class WatchFileModifyListener implements Runnable
    {

        @Override
        public void run()
        {
            while (true)
            {
                sleep(5000);
                // 5s 检测一次文件
                if (null == filePath || "".equals(filePath))
                {
                    return;
                }
                File checkFile = new File(filePath);
                if (lastModified != checkFile.lastModified())
                {
                    loadConfigFile(filePath);
                    // 将更改的文件拷贝到系统中
                    WriteFile.copyFile(checkFile, new File(FILE_PATH + FILE_NAME));
                    lastModified = checkFile.lastModified();
                    Log.d(TAG, "file has modified");
                    notifyConfigChange();
                }
            }
        }
        
    }
    
}
